package java.lang;

import java.io.ObjectStreamException;
import java.io.IOException;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.io.Serializable;

public abstract class Enum<E extends Enum<E>> implements Comparable<E>, Serializable
{
    private final String name;
    private final int ordinal;
    
    public final String name() {
        return this.name;
    }
    
    public final int ordinal() {
        return this.ordinal;
    }
    
    protected Enum(String name, int ordinal) {
        this.name = name;
        this.ordinal = ordinal;
    }
    
    @Override
    public String toString() {
        return this.name;
    }
    
    @Override
    public final boolean equals(Object o) {
        return this == o;
    }
    
    @Override
    public final int hashCode() {
        return super.hashCode();
    }
    
    @Override
    protected final Object clone() throws CloneNotSupportedException {
        throw new CloneNotSupportedException();
    }
    
    @Override
    public final int compareTo(E e) {
        if (this.getClass() != e.getClass() && this.getDeclaringClass() != e.getDeclaringClass()) {
            throw new ClassCastException();
        }
        return this.ordinal - e.ordinal;
    }
    
    public final Class<E> getDeclaringClass() {
        Class<Object> class1 = this.getClass();
        Class<Object> superclass = class1.getSuperclass();
        return (superclass == Enum.class) ? class1 : superclass;
    }
    
    public static <T extends Enum<T>> T valueOf(Class<T> clazz, String s) {
        Enum<Object> enum1 = (T)clazz.enumConstantDirectory().get(s);
        if (enum1 != null) {
            return enum1;
        }
        if (s == null) {
            throw new NullPointerException("Name is null");
        }
        throw new IllegalArgumentException("No enum constant " + clazz.getCanonicalName() + "." + s);
    }
    
    @Override
    protected final void finalize() {
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        throw new InvalidObjectException("can't deserialize enum");
    }
    
    private void readObjectNoData() throws ObjectStreamException {
        throw new InvalidObjectException("can't deserialize enum");
    }
}
